@@ -136,7 +136,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
             'wrong_method': 'Wrong combination of method!',
             'addresses_required': 'Invoice and Shipment addresses must be '
             'defined for the quotation.',
-            'missing_account_receivable': 'It misses ' 
+            'missing_account_receivable': 'It misses '
                     'an "Account Receivable" on the party "%s"!',
         })
 
@@ -209,13 +209,17 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         return 2
 
     def default_invoice_method(self):
-        return 'order'
+        config_obj = self.pool.get('sale.configuration')
+        config = config_obj.browse(1)
+        return config.sale_invoice_method
 
     def default_invoice_state(self):
         return 'none'
 
     def default_shipment_method(self):
-        return 'order'
+        config_obj = self.pool.get('sale.configuration')
+        config = config_obj.browse(1)
+        return config.sale_shipment_method
 
     def default_shipment_state(self):
         return 'none'
@@ -231,9 +235,9 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         }
         if vals.get('party'):
             party = party_obj.browse(vals['party'])
-            res['invoice_address'] = party_obj.address_get(party.id, 
+            res['invoice_address'] = party_obj.address_get(party.id,
                     type='invoice')
-            res['shipment_address'] = party_obj.address_get(party.id, 
+            res['shipment_address'] = party_obj.address_get(party.id,
                     type='delivery')
             if party.payment_term:
                 res['payment_term'] = party.payment_term.id
@@ -343,7 +347,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
                         taxes[key] += val['amount']
             if currency:
                 for key in taxes:
-                    res['tax_amount'] += currency_obj.round(currency, 
+                    res['tax_amount'] += currency_obj.round(currency,
                             taxes[key])
         if currency:
             res['untaxed_amount'] = currency_obj.round(currency,
@@ -650,7 +654,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
 
         :param sale: the BrowseRecord of the sale
 
-        :return: a dictionary with invoiced sale line id as key
+        :return: a dictionary with invoice line as key
             and a list of invoice lines values as value
         '''
         line_obj = self.pool.get('sale.line')
@@ -718,14 +722,12 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         with Transaction().set_user(0, set_context=True):
             invoice_id = invoice_obj.create(vals)
 
-        for line in sale.lines:
-            if line.id not in invoice_lines:
-                continue
-            for vals in invoice_lines[line.id]:
+        for line_id in invoice_lines:
+            for vals in invoice_lines[line_id]:
                 vals['invoice'] = invoice_id
                 with Transaction().set_user(0, set_context=True):
                     invoice_line_id = invoice_line_obj.create(vals)
-                sale_line_obj.write(line.id, {
+                sale_line_obj.write(line_id, {
                     'invoice_lines': [('add', invoice_line_id)],
                     })
 
@@ -896,7 +898,7 @@ class SaleLine(ModelSQL, ModelView):
         states={
             'invisible': Not(In(Eval('type'), ['line', 'subtotal'])),
             'readonly': Not(Bool(Eval('_parent_sale'))),
-        }, on_change_with=['type', 'quantity', 'unit_price', 'unit',
+        }, on_change_with=['type', 'quantity', 'unit_price',
             '_parent_sale.currency']), 'get_amount')
     description = fields.Text('Description', size=None, required=True)
     note = fields.Text('Note')
@@ -981,7 +983,7 @@ class SaleLine(ModelSQL, ModelView):
                         and move.id not in skip_ids:
                     val = False
                     break
-                quantity -= uom_obj.compute_qty(move.uom, move.quantity, 
+                quantity -= uom_obj.compute_qty(move.uom, move.quantity,
                         line.unit)
             if val:
                 if quantity > 0.0:
@@ -1020,6 +1022,8 @@ class SaleLine(ModelSQL, ModelView):
             context['currency'] = vals['_parent_sale.currency']
         if vals.get('_parent_sale.party'):
             context['customer'] = vals['_parent_sale.party']
+        if vals.get('_parent_sale.sale_date'):
+            context['sale_date'] = vals['_parent_sale.sale_date']
         if vals.get('unit'):
             context['uom'] = vals['unit']
         else:
@@ -1047,20 +1051,20 @@ class SaleLine(ModelSQL, ModelView):
 
         with Transaction().set_context(
                 self._get_context_sale_price(product,vals)):
-            res['unit_price'] = product_obj.get_sale_price([product.id], 
+            res['unit_price'] = product_obj.get_sale_price([product.id],
                     vals.get('quantity', 0))[product.id]
         res['taxes'] = []
         pattern = self._get_tax_rule_pattern(party, vals)
         for tax in product.customer_taxes_used:
             if party and party.customer_tax_rule:
-                tax_ids = tax_rule_obj.apply(party.customer_tax_rule, tax, 
+                tax_ids = tax_rule_obj.apply(party.customer_tax_rule, tax,
                         pattern)
                 if tax_ids:
                     res['taxes'].extend(tax_ids)
                 continue
             res['taxes'].append(tax.id)
         if party and party.customer_tax_rule:
-            tax_ids = tax_rule_obj.apply(party.customer_tax_rule, False, 
+            tax_ids = tax_rule_obj.apply(party.customer_tax_rule, False,
                     pattern)
             if tax_ids:
                 res['taxes'].extend(tax_ids)
@@ -1093,7 +1097,7 @@ class SaleLine(ModelSQL, ModelView):
 
         with Transaction().set_context(
                 self._get_context_sale_price(product, vals)):
-            res['unit_price'] = product_obj.get_sale_price([vals['product']], 
+            res['unit_price'] = product_obj.get_sale_price([vals['product']],
                     vals.get('quantity', 0))[vals['product']]
         return res
 
@@ -1103,10 +1107,9 @@ class SaleLine(ModelSQL, ModelView):
     def on_change_with_amount(self, vals):
         currency_obj = self.pool.get('currency.currency')
         if vals.get('type') == 'line':
-            if isinstance(vals.get('_parent_sale.currency'), (int, long)):
+            currency = vals.get('_parent_sale.currency')
+            if currency and isinstance(currency, (int, long)):
                 currency = currency_obj.browse(vals['_parent_sale.currency'])
-            else:
-                currency = vals['_parent_sale.currency']
             amount = Decimal(str(vals.get('quantity') or '0.0')) * \
                     (vals.get('unit_price') or Decimal('0.0'))
             if currency:
@@ -1126,7 +1129,7 @@ class SaleLine(ModelSQL, ModelView):
                 for line2 in line.sale.lines:
                     if line2.type == 'line':
                         res[line.id] += currency_obj.round(line2.sale.currency,
-                                Decimal(str(line2.quantity)) * 
+                                Decimal(str(line2.quantity)) *
                                         line2.unit_price)
                     elif line2.type == 'subtotal':
                         if line.id == line2.id:
@@ -1155,26 +1158,22 @@ class SaleLine(ModelSQL, ModelView):
         if line.type != 'line':
             return [res]
 
-        if (line.sale.invoice_method == 'order'
-                or not line.product
-                or line.product.type == 'service'):
+        if line.sale.invoice_method == 'order':
             quantity = line.quantity
         else:
             quantity = 0.0
             for move in line.moves:
                 if move.state == 'done':
-                    quantity += uom_obj.compute_qty(move.uom, move.quantity, 
+                    quantity += uom_obj.compute_qty(move.uom, move.quantity,
                             line.unit)
 
         ignored_ids = set(
             l.id for i in line.sale.invoices_ignored for l in i.lines)
         for invoice_line in line.invoice_lines:
-            if invoice_line.type != 'line':
-                continue
             if ((invoice_line.invoice and
                     invoice_line.invoice.state != 'cancel') or
                 invoice_line.id in ignored_ids):
-                quantity -= uom_obj.compute_qty(invoice_line.unit, 
+                quantity -= uom_obj.compute_qty(invoice_line.unit,
                         invoice_line.quantity, line.unit)
         res['quantity'] = quantity
 
@@ -1386,7 +1385,9 @@ class Product(ModelSQL, ModelView):
         uom_obj = self.pool.get('product.uom')
         user_obj = self.pool.get('res.user')
         currency_obj = self.pool.get('currency.currency')
+        date_obj = self.pool.get('ir.date')
 
+        today = date_obj.today()
         res = {}
 
         uom = None
@@ -1407,8 +1408,11 @@ class Product(ModelSQL, ModelView):
                         product.default_uom, res[product.id], uom)
             if currency and user2.company:
                 if user2.company.currency.id != currency.id:
-                    res[product.id] = currency_obj.compute(
-                            user2.company.currency, res[product.id], currency)
+                    date = Transaction().context.get('sale_date') or today
+                    with Transaction().set_context(date=date):
+                        res[product.id] = currency_obj.compute(
+                                user2.company.currency.id, res[product.id],
+                                currency.id)
         return res
 
 Product()
@@ -1598,7 +1602,7 @@ class Invoice(ModelSQL, ModelView):
             return True
         if isinstance(ids, (int, long)):
             ids = [ids]
-        Transaction().cursor.execute('SELECT id FROM sale_invoices_rel ' 
+        Transaction().cursor.execute('SELECT id FROM sale_invoices_rel '
                 'WHERE invoice IN (' + ','.join(('%s',) * len(ids)) + ')',
                 ids)
         if Transaction().cursor.fetchone():
